package com.amap.api.maps.utils.overlay;

import com.amap.api.maps.model.CameraPosition;
import com.autonavi.amap.mapcore.MapProjection;
import com.autonavi.amap.mapcore.IPoint;
import com.amap.api.col.3sl.lz;
import com.amap.api.maps.model.animation.Animation;
import java.util.concurrent.TimeUnit;
import java.util.Iterator;
import com.amap.api.maps.model.MarkerOptions;
import com.amap.api.maps.AMapUtils;
import java.util.List;
import com.amap.api.col.3sl.dx;
import java.util.concurrent.atomic.AtomicBoolean;
import com.amap.api.maps.model.BitmapDescriptor;
import com.amap.api.maps.model.Marker;
import com.amap.api.col.3sl.ly;
import com.amap.api.maps.model.LatLng;
import java.util.LinkedList;
import com.amap.api.maps.AMap;

public class SmoothMoveMarker
{
    public static final float MIN_OFFSET_DISTANCE = 5.0f;
    private AMap mAMap;
    private long duration;
    private long mStepDuration;
    private LinkedList<LatLng> points;
    private LinkedList<Double> eachDistance;
    private double totalDistance;
    private double remainDistance;
    private ly mThreadPools;
    private Object mLock;
    private Marker marker;
    private BitmapDescriptor descriptor;
    private int index;
    private boolean useDefaultDescriptor;
    AtomicBoolean exitFlag;
    private SmoothMoveMarker.MoveListener moveListener;
    private SmoothMoveMarker.a moveStatus;
    private long pauseMillis;
    private long mAnimationBeginTime;
    
    public SmoothMoveMarker(final AMap maMap) {
        this.duration = 10000L;
        this.mStepDuration = 20L;
        this.points = new LinkedList<LatLng>();
        this.eachDistance = new LinkedList<Double>();
        this.totalDistance = 0.0;
        this.remainDistance = 0.0;
        this.mLock = new Object();
        this.marker = null;
        this.index = 0;
        this.useDefaultDescriptor = false;
        this.exitFlag = new AtomicBoolean(false);
        this.moveStatus = SmoothMoveMarker.a.a;
        this.mAnimationBeginTime = System.currentTimeMillis();
        this.mAMap = maMap;
        this.mThreadPools = dx.a("AMapSmoothMoveMarkerThread");
    }
    
    public void setPoints(final List<LatLng> list) {
        synchronized (this.mLock) {
            try {
                if (list == null || list.size() < 2) {
                    return;
                }
                this.stopMove();
                this.points.clear();
                final Iterator iterator = list.iterator();
                while (iterator.hasNext()) {
                    final LatLng e;
                    if ((e = (LatLng)iterator.next()) != null) {
                        this.points.add(e);
                    }
                }
                this.eachDistance.clear();
                this.totalDistance = 0.0;
                for (int i = 0; i < this.points.size() - 1; ++i) {
                    final double d = AMapUtils.calculateLineDistance((LatLng)this.points.get(i), (LatLng)this.points.get(i + 1));
                    this.eachDistance.add(d);
                    this.totalDistance += d;
                }
                this.remainDistance = this.totalDistance;
                final LatLng position = this.points.get(0);
                if (this.marker != null) {
                    this.marker.setPosition(position);
                    this.checkMarkerIcon();
                }
                else {
                    if (this.descriptor == null) {
                        this.useDefaultDescriptor = true;
                    }
                    this.marker = this.mAMap.addMarker(new MarkerOptions().belowMaskLayer(true).position(position).icon(this.descriptor).title("").anchor(0.5f, 0.5f));
                }
                this.reset();
            }
            catch (final Throwable t) {
                t.printStackTrace();
            }
        }
    }
    
    private void reset() {
        try {
            if (this.moveStatus == SmoothMoveMarker.a.c || this.moveStatus == SmoothMoveMarker.a.d) {
                this.exitFlag.set(true);
                this.mThreadPools.a(this.mStepDuration + 20L, TimeUnit.MILLISECONDS);
                if (this.marker != null) {
                    this.marker.setAnimation((Animation)null);
                }
                this.moveStatus = SmoothMoveMarker.a.a;
            }
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    private void checkMarkerIcon() {
        if (this.useDefaultDescriptor) {
            if (this.descriptor == null) {
                this.useDefaultDescriptor = true;
                return;
            }
            this.marker.setIcon(this.descriptor);
            this.useDefaultDescriptor = false;
        }
    }
    
    public void setTotalDuration(final int n) {
        this.duration = n * 1000;
    }
    
    public void startSmoothMove() {
        if (this.moveStatus == SmoothMoveMarker.a.d) {
            this.moveStatus = SmoothMoveMarker.a.c;
            this.mAnimationBeginTime += System.currentTimeMillis() - this.pauseMillis;
            return;
        }
        if (this.moveStatus == SmoothMoveMarker.a.a || this.moveStatus == SmoothMoveMarker.a.e) {
            if (this.points.size() <= 0) {
                return;
            }
            this.index = 0;
            try {
                this.mThreadPools.a((lz)new SmoothMoveMarker.b(this, (byte)0));
            }
            catch (final Throwable t) {
                t.printStackTrace();
            }
        }
    }
    
    private IPoint getCurPosition(final long n) {
        if (n > this.duration) {
            this.exitFlag.set(true);
            final IPoint point = new IPoint();
            this.index = this.points.size() - 1;
            final LatLng latLng = this.points.get(this.index);
            --this.index;
            this.index = Math.max(this.index, 0);
            this.remainDistance = 0.0;
            MapProjection.lonlat2Geo(latLng.longitude, latLng.latitude, point);
            if (this.moveListener != null) {
                this.moveListener.move(this.remainDistance);
            }
            return point;
        }
        double n2 = n * this.totalDistance / this.duration;
        this.remainDistance = this.totalDistance - n2;
        int n3 = 0;
        double n4 = 1.0;
        for (int i = 0; i < this.eachDistance.size(); ++i) {
            final double doubleValue = this.eachDistance.get(i);
            if (n2 <= doubleValue) {
                if (doubleValue > 0.0) {
                    n4 = n2 / doubleValue;
                }
                n3 = i;
                break;
            }
            n2 -= doubleValue;
        }
        if (n3 != this.index && this.moveListener != null) {
            this.moveListener.move(this.remainDistance);
        }
        this.index = n3;
        final LatLng latLng2 = this.points.get(n3);
        final LatLng latLng3 = this.points.get(n3 + 1);
        final IPoint point2 = new IPoint();
        MapProjection.lonlat2Geo(latLng2.longitude, latLng2.latitude, point2);
        final IPoint point3 = new IPoint();
        MapProjection.lonlat2Geo(latLng3.longitude, latLng3.latitude, point3);
        final int n5 = point3.x - point2.x;
        final int n6 = point3.y - point2.y;
        if (AMapUtils.calculateLineDistance(latLng2, latLng3) > 5.0f) {
            final float rotate = this.getRotate(point2, point3);
            final CameraPosition cameraPosition;
            if (this.mAMap != null && (cameraPosition = this.mAMap.getCameraPosition()) != null) {
                this.marker.setRotateAngle(360.0f - rotate + cameraPosition.bearing);
            }
        }
        return new IPoint((int)(point2.x + n5 * n4), (int)(point2.y + n6 * n4));
    }
    
    private float getRotate(final IPoint point, final IPoint point2) {
        if (point == null || point2 == null) {
            return 0.0f;
        }
        return (float)(Math.atan2(point2.x - (double)point.x, point.y - (double)point2.y) / 3.141592653589793 * 180.0);
    }
    
    public void stopMove() {
        if (this.moveStatus == SmoothMoveMarker.a.c) {
            this.moveStatus = SmoothMoveMarker.a.d;
            this.pauseMillis = System.currentTimeMillis();
        }
    }
    
    public Marker getMarker() {
        return this.marker;
    }
    
    public LatLng getPosition() {
        if (this.marker == null) {
            return null;
        }
        return this.marker.getPosition();
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public void resetIndex() {
        this.index = 0;
    }
    
    public void destroy() {
        try {
            this.reset();
            this.mThreadPools.e();
            if (this.descriptor != null) {
                this.descriptor.recycle();
            }
            if (this.marker != null) {
                this.marker.destroy();
                this.marker = null;
            }
            synchronized (this.mLock) {
                this.points.clear();
                this.eachDistance.clear();
            }
        }
        catch (final Throwable t) {
            t.printStackTrace();
        }
    }
    
    public void removeMarker() {
        if (this.marker != null) {
            this.marker.remove();
            this.marker = null;
        }
        this.points.clear();
        this.eachDistance.clear();
    }
    
    public void setPosition(final LatLng position) {
        if (this.marker != null) {
            this.marker.setPosition(position);
            this.checkMarkerIcon();
            return;
        }
        if (this.descriptor == null) {
            this.useDefaultDescriptor = true;
        }
        this.marker = this.mAMap.addMarker(new MarkerOptions().belowMaskLayer(true).position(position).icon(this.descriptor).title("").anchor(0.5f, 0.5f));
    }
    
    public void setDescriptor(final BitmapDescriptor bitmapDescriptor) {
        if (this.descriptor != null) {
            this.descriptor.recycle();
        }
        this.descriptor = bitmapDescriptor;
        if (this.marker != null) {
            this.marker.setIcon(bitmapDescriptor);
        }
    }
    
    public void setRotate(final float n) {
        final CameraPosition cameraPosition;
        if (this.marker != null && this.mAMap != null && this.mAMap != null && (cameraPosition = this.mAMap.getCameraPosition()) != null) {
            this.marker.setRotateAngle(360.0f - n + cameraPosition.bearing);
        }
    }
    
    public void setVisible(final boolean visible) {
        if (this.marker != null) {
            this.marker.setVisible(visible);
        }
    }
    
    public void setMoveListener(final SmoothMoveMarker.MoveListener moveListener) {
        this.moveListener = moveListener;
    }
}
